/*
	framebuf.c

	original code: SASANO Takayoshi
	--- no warranty, public domain.
*/

#include <stdio.h>
#include <kernel/segment.h>
#include <device/screen.h>
#include <btron/device.h>
#include <btron/dp.h>
#include <tcode.h>
#include <errcode.h>

#ifdef	DEBUG
#define	D(x)	printf x
#else
#define	D(x)	/* */
#endif

#define	SAFETY_MARGIN	0x00100000

/* get information of frame buffer */
EXPORT	ERR	getinfo_fb(VP *fbaddr, W *fbsize, W *usedsize)
{
	ERR	err;
	W	dd, rsz;
	BMP	bmp;
	ScrDevInfo	scrdevinfo;
	LOCAL	TC	devname[] = {
		TK_S, TK_C, TK_R, TK_E, TK_E, TK_N, TK_NULL,
	};

	err = b_opn_dev(devname, D_READ, NULL);
	if (err < ER_OK) {
		D(("get_fbinfo: b_opn_dev %d\n", err));
		goto fin0;
	}
	dd = err;

	err = b_rea_dev(dd, DN_SCRBMP, (UB *)&bmp,
			sizeof(bmp), &rsz, NULL);
	if (err < ER_OK) {
		D(("get_fbinfo: b_rea_dev %d\n", err));
		goto fin1;
	}

	err = b_rea_dev(dd, DN_SCRDEVINFO, (UB *)&scrdevinfo,
			sizeof(scrdevinfo), &rsz, NULL);
	if (err < ER_OK) {
		D(("get_fbinfo: b_rea_dev %d\n", err));
		goto fin1;
	}

	D(("[DN_SCRBMP]\n"));
	D(("planes       : %d\n",   bmp.planes));
	D(("pixbits      : %08x\n", bmp.pixbits));
	D(("rowbytes     : %d\n",   bmp.rowbytes));
	D(("bounds       : (%d, %d) - (%d, %d)\n",
	   bmp.bounds.c.left,  bmp.bounds.c.top,
	   bmp.bounds.c.right, bmp.bounds.c.bottom));
	D(("baseaddr[0]  : %08x\n", bmp.baseaddr[0]));

	D(("[DN_SCRDEVINFO]\n"));
	D(("name1        : %s\n",   scrdevinfo.name1));
	D(("name2        : %s\n",   scrdevinfo.name2));
	D(("name3        : %s\n",   scrdevinfo.name3));
	D(("framebuf_addr: %08x\n", scrdevinfo.framebuf_addr));
	D(("framebuf_size: %08x\n", scrdevinfo.framebuf_size));
	D(("mainmem_size : %08x\n", scrdevinfo.mainmem_size));

	if (scrdevinfo.framebuf_addr) {
		*fbaddr = scrdevinfo.framebuf_addr;
		*fbsize = scrdevinfo.framebuf_size;
	} else {
		*fbaddr = NULL;
		*fbsize = 0;
	}

	*usedsize = (bmp.rowbytes * 
		     (bmp.bounds.c.bottom - bmp.bounds.c.top));

	/* safety margin (at least specified size) */
	*usedsize += SAFETY_MARGIN * 2 - 1;
	*usedsize &= ~(SAFETY_MARGIN - 1);
	
	err = ER_OK;

 fin1:
	b_cls_dev(dd, 0, NULL);
 fin0:
	return err;
}

/* map framebuffer */
EXPORT	ERR	map_fb(VP paddr, W len, VP *laddr)
{
	return MapMemory(paddr, len,
			 MM_SYSTEM | MM_READ | MM_WRITE | MM_CDIS, laddr);
}

/* unmap framebuffer */
EXPORT	ERR	unmap_fb(VP laddr)
{
	return UnmapMemory(laddr);
}
